@@ -27,7 +27,8 @@ def shell(cmd: List[str]) -> Optional[str]:
             .decode("utf8")
             .strip()
         )
-    except subprocess.CalledProcessError:
+    except subprocess.CalledProcessError as e:
+        print(e)
         return None
 
 
@@ -206,6 +207,26 @@ def image_id(image_name: str) -> Optional[str]:
         ) or image_id_from_registry(image_name)
 
 
+def get_image_uid(image_name: str) -> int:
+    """Retreve the image default uid through brute force"""
+    return int(shell(["run", image_name, "id", "-u"]))
+
+
+def push(image: str, tag: str) -> Optional[str]:
+    """Push an image to a remote registry"""
+    return shell(["push", f"{image}:{tag}"])
+
+
+def login(username: str, password: str, registry: str) -> Optional[str]:
+    """Login to a registry"""
+    return shell(["login", "--username", username, "--password", password, registry])
+
+
+def tag(image_name: str, tag: str) -> Optional[str]:
+    """Tag an image"""
+    return shell(["tag", image_name, tag])
+
+
 __all__ = [
     "shell",
     "build",
@@ -216,4 +237,8 @@ __all__ = [
     "parse",
     "parse_repository_tag",
     "default_image",
+    "get_image_uid",
+    "push",
+    "login",
+    "tag",
 ]
