@@ -3,11 +3,11 @@ from collections import OrderedDict
 import re
 
 _tokens = (
-    ('token', re.compile(r'''^([!#$%&'*+\-.^_`|~\w/]+(?:={1,2}$)?)''')),
-    ('token', re.compile(r'''^"((?:[^"\\]|\\\\|\\")+)"''')),
-    (None, re.compile(r'^\s+')),
-    ('equals', re.compile(r'^(=)')),
-    ('comma', re.compile(r'^(,)')),
+    ("token", re.compile(r"""^([!#$%&'*+\-.^_`|~\w/]+(?:={1,2}$)?)""")),
+    ("token", re.compile(r'''^"((?:[^"\\]|\\\\|\\")+)"''')),
+    (None, re.compile(r"^\s+")),
+    ("equals", re.compile(r"^(=)")),
+    ("comma", re.compile(r"^(,)")),
 )
 
 
@@ -38,10 +38,12 @@ class CaseFoldedOrderedDict(OrderedDict):
 def _group_pairs(tokens):
     i = 0
     while i < len(tokens) - 2:
-        if tokens[i][0] == 'token' and \
-           tokens[i+1][0] == 'equals' and \
-           tokens[i+2][0] == 'token':
-            tokens[i:i+3] = [('pair', (tokens[i][1], tokens[i+2][1]))]
+        if (
+            tokens[i][0] == "token"
+            and tokens[i + 1][0] == "equals"
+            and tokens[i + 2][0] == "token"
+        ):
+            tokens[i : i + 3] = [("pair", (tokens[i][1], tokens[i + 2][1]))]
         i += 1
 
 
@@ -51,16 +53,16 @@ def _group_challenges(tokens):
         j = 1
         if len(tokens) == 1:
             pass
-        elif tokens[1][0] == 'comma':
+        elif tokens[1][0] == "comma":
             pass
-        elif tokens[1][0] == 'token':
+        elif tokens[1][0] == "token":
             j = 2
         else:
-            while j < len(tokens) and tokens[j][0] == 'pair':
+            while j < len(tokens) and tokens[j][0] == "pair":
                 j += 2
             j -= 1
         challenges.append((tokens[0][1], tokens[1:j]))
-        tokens[:j+1] = []
+        tokens[: j + 1] = []
     return challenges
 
 
@@ -70,7 +72,7 @@ def parse(value):
         for token_name, pattern in _tokens:
             match = pattern.match(value)
             if match:
-                value = value[match.end():]
+                value = value[match.end() :]
                 if token_name:
                     tokens.append((token_name, match.group(1)))
                 break
@@ -79,12 +81,12 @@ def parse(value):
     _group_pairs(tokens)
 
     challenges = CaseFoldedOrderedDict()
-    for name, tokens in _group_challenges(tokens):
+    for name, tokens in _group_challenges(tokens):  # noqa: B020
         args, kwargs = [], {}
         for token_name, value in tokens:
-            if token_name == 'token':
+            if token_name == "token":
                 args.append(value)
-            elif token_name == 'pair':
+            elif token_name == "pair":
                 kwargs[value[0]] = value[1]
         challenges[name] = (args and args[0]) or kwargs or None
 
