@@ -49,6 +49,7 @@ HTTP_TIMEOUT = "WANDB_HTTP_TIMEOUT"
 API_KEY = "WANDB_API_KEY"
 JOB_TYPE = "WANDB_JOB_TYPE"
 DISABLE_CODE = "WANDB_DISABLE_CODE"
+DISABLE_GIT = "WANDB_DISABLE_GIT"
 SAVE_CODE = "WANDB_SAVE_CODE"
 TAGS = "WANDB_TAGS"
 IGNORE = "WANDB_IGNORE_GLOBS"
@@ -66,6 +67,8 @@ JUPYTER = "WANDB_JUPYTER"
 CONFIG_DIR = "WANDB_CONFIG_DIR"
 CACHE_DIR = "WANDB_CACHE_DIR"
 DISABLE_SSL = "WANDB_INSECURE_DISABLE_SSL"
+SERVICE = "WANDB_SERVICE"
+SENTRY_DSN = "WANDB_SENTRY_DSN"
 
 # For testing, to be removed in future version
 USE_V1_ARTIFACTS = "_WANDB_USE_V1_ARTIFACTS"
@@ -81,6 +84,7 @@ def immutable_keys():
         API_KEY,
         IGNORE,
         DISABLE_CODE,
+        DISABLE_GIT,
         DOCKER,
         MODE,
         BASE_URL,
@@ -106,7 +110,7 @@ def immutable_keys():
     ]
 
 
-def _env_as_bool(var, default=None, env=None):
+def _env_as_bool(var: str, default: str = None, env=None) -> bool:
     if env is None:
         env = os.environ
     val = env.get(var, default)
@@ -121,7 +125,7 @@ def is_debug(default=None, env=None):
     return _env_as_bool(DEBUG, default=default, env=env)
 
 
-def error_reporting_enabled():
+def error_reporting_enabled() -> bool:
     return _env_as_bool(ERROR_REPORTING, default=True)
 
 
@@ -211,7 +215,7 @@ def get_base_url(default=None, env=None):
     if env is None:
         env = os.environ
 
-    return env.get(BASE_URL, default)
+    return env.get(BASE_URL, default).rstrip("/")
 
 
 def get_app_url(default=None, env=None):
@@ -335,5 +339,13 @@ def set_project(value, env=None):
 def should_save_code():
     save_code = _env_as_bool(SAVE_CODE, default=False)
     code_disabled = _env_as_bool(DISABLE_CODE, default=False)
-    # SAVE_CODE takes precedence over DISABLE_CODE
     return save_code and not code_disabled
+
+
+def disable_git(env=None):
+    if env is None:
+        env = os.environ
+    val = env.get(DISABLE_GIT, default=False)
+    if type(val) is str:
+        val = False if val.lower() == "false" else True
+    return val
