@@ -11,7 +11,7 @@ import six
 
 import wandb
 from wandb import util
-from wandb import data_types
+from wandb.sdk.data_types.utils import val_to_json
 from wandb.apis.internal import Api
 from six import string_types
 
@@ -327,7 +327,7 @@ class Summary(SummarySubDict):
         else:
             path = ".".join(path_from_root)
             friendly_value, converted = util.json_friendly(
-                data_types.val_to_json(self._run, path, value, namespace="summary"))
+                val_to_json(self._run, path, value, namespace="summary"))
             json_value, compressed = util.maybe_compress_summary(
                 friendly_value, util.get_h5_typename(value))
             if compressed:
