@@ -5,27 +5,25 @@ import contextlib
 import hashlib
 import os
 import random
+from typing import (
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Sequence,
+    Tuple,
+    TYPE_CHECKING,
+    Union,
+)
 
 import wandb
 from wandb import env
 from wandb import util
 from wandb.data_types import WBValue
 
-if wandb.TYPE_CHECKING:  # type: ignore
 
-    from typing import (
-        List,
-        Optional,
-        Union,
-        Dict,
-        Callable,
-        TYPE_CHECKING,
-        Sequence,
-        Tuple,
-    )
-
-    if TYPE_CHECKING:
-        import wandb.filesync.step_prepare.StepPrepare as StepPrepare  # type: ignore
+if TYPE_CHECKING:
+    import wandb.filesync.step_prepare.StepPrepare as StepPrepare  # type: ignore
 
 
 def md5_string(string: str) -> str:
@@ -46,10 +44,26 @@ def md5_hash_file(path):
     return hash_md5
 
 
+def md5_hash_files(paths: List[str]):
+    hash_md5 = hashlib.md5()
+    # Create a mutable copy to sort
+    paths = [path for path in paths]
+    paths.sort()
+    for path in paths:
+        with open(path, "rb") as f:
+            for chunk in iter(lambda: f.read(64 * 1024), b""):
+                hash_md5.update(chunk)
+    return hash_md5
+
+
 def md5_file_b64(path: str) -> str:
     return base64.b64encode(md5_hash_file(path).digest()).decode("ascii")
 
 
+def md5_files_b64(paths: List[str]) -> str:
+    return base64.b64encode(md5_hash_files(paths).digest()).decode("ascii")
+
+
 def md5_file_hex(path: str) -> str:
     return md5_hash_file(path).hexdigest()
 
@@ -64,13 +78,14 @@ class ArtifactManifest(object):
 
     @classmethod
     # TODO: we don't need artifact here.
-    def from_manifest_json(cls, artifact, manifest_json):
+    def from_manifest_json(cls, artifact, manifest_json) -> "ArtifactManifest":
         if "version" not in manifest_json:
             raise ValueError("Invalid manifest format. Must contain version field.")
         version = manifest_json["version"]
         for sub in cls.__subclasses__():
             if sub.version() == version:
                 return sub.from_manifest_json(artifact, manifest_json)
+        raise ValueError("Invalid manifest version.")
 
     @classmethod
     def version(cls):
@@ -647,6 +662,21 @@ class Artifact(object):
         """
         raise NotImplementedError
 
+    def link(self, target_path: str, aliases: Optional[List[str]] = None) -> None:
+        """
+        Links this artifact to a portfolio (a promoted collection of artifacts), with aliases.
+
+        Arguments:
+            target_path: (str) The path to the portfolio. It must take the form
+                {portfolio}, {project}/{portfolio} or {entity}/{project}/{portfolio}.
+            aliases: (Optional[List[str]]) A list of strings which uniquely
+                identifies the artifact inside the specified portfolio.
+
+        Returns:
+            None
+        """
+        raise NotImplementedError
+
     def delete(self) -> None:
         """
         Deletes this artifact, cleaning up all files associated with it.
@@ -833,6 +863,7 @@ class ArtifactsCache(object):
         self._artifacts_by_id = {}
         self._random = random.Random()
         self._random.seed()
+        self._artifacts_by_client_id = {}
 
     def check_md5_obj_path(self, b64_md5: str, size: int) -> Tuple[str, bool, Callable]:
         hex_md5 = util.bytes_to_hex(base64.b64decode(b64_md5))
@@ -857,6 +888,12 @@ class ArtifactsCache(object):
     def store_artifact(self, artifact):
         self._artifacts_by_id[artifact.id] = artifact
 
+    def get_client_artifact(self, client_id):
+        return self._artifacts_by_client_id.get(client_id)
+
+    def store_client_artifact(self, artifact):
+        self._artifacts_by_client_id[artifact._client_id] = artifact
+
     def cleanup(self, target_size: int) -> int:
         bytes_reclaimed: int = 0
         paths: Dict[os.PathLike, os.stat_result] = {}
