@@ -3,7 +3,7 @@ import json
 import os
 import tempfile
 import threading
-from typing import Dict, List, Optional, TYPE_CHECKING
+from typing import Dict, List, Optional, Sequence, TYPE_CHECKING
 
 import wandb
 from wandb import util
@@ -79,10 +79,11 @@ class ArtifactSaver(object):
         finalize: bool = True,
         metadata: Optional[Dict] = None,
         description: Optional[str] = None,
-        aliases: Optional[List[str]] = None,
+        aliases: Optional[Sequence[str]] = None,
         labels: Optional[List[str]] = None,
         use_after_commit: bool = False,
         incremental: bool = False,
+        history_step: Optional[int] = None,
     ) -> Optional[Dict]:
         aliases = aliases or []
         alias_specs = []
@@ -114,6 +115,8 @@ class ArtifactSaver(object):
             distributed_id=distributed_id,
             client_id=client_id,
             sequence_client_id=sequence_client_id,
+            enable_digest_deduplication=use_after_commit,  # Reuse logical duplicates in the `use_artifact` flow
+            history_step=history_step,
         )
 
         # TODO(artifacts):
