@@ -964,13 +964,39 @@ class SendManager:
         # tbrecord watching threads are handled by handler.py
         pass
 
+    def send_link_artifact(self, record: "Record") -> None:
+        link = record.link_artifact
+        client_id = link.client_id
+        server_id = link.server_id
+        portfolio_name = link.portfolio_name
+        entity = link.portfolio_entity
+        project = link.portfolio_project
+        aliases = link.portfolio_aliases
+        logger.debug(
+            f"link_artifact params - client_id={client_id}, server_id={server_id}, pfolio={portfolio_name}, entity={entity}, project={project}"
+        )
+        if (
+            (client_id or server_id)
+            and portfolio_name
+            and entity
+            and project
+            and aliases
+        ):
+            try:
+                self._api.link_artifact(
+                    client_id, server_id, portfolio_name, entity, project, aliases
+                )
+            except Exception as e:
+                logger.warning("Failed to link artifact to portfolio: %s", e)
+
     def send_request_log_artifact(self, record: "Record") -> None:
         assert record.control.req_resp
         result = proto_util._result_from_record(record)
         artifact = record.request.log_artifact.artifact
+        history_step = record.request.log_artifact.history_step
 
         try:
-            res = self._send_artifact(artifact)
+            res = self._send_artifact(artifact, history_step)
             assert res, "Unable to send artifact"
             result.response.log_artifact_response.artifact_id = res["id"]
             logger.info("logged artifact {} - {}".format(artifact.name, res))
@@ -1015,7 +1041,9 @@ class SendManager:
                 )
             )
 
-    def _send_artifact(self, artifact: "ArtifactRecord") -> Optional[Dict]:
+    def _send_artifact(
+        self, artifact: "ArtifactRecord", history_step: Optional[int] = None
+    ) -> Optional[Dict]:
         assert self._pusher
         saver = artifacts.ArtifactSaver(
             api=self._api,
@@ -1049,6 +1077,7 @@ class SendManager:
             distributed_id=artifact.distributed_id,
             finalize=artifact.finalize,
             incremental=artifact.incremental_beta1,
+            history_step=history_step,
         )
 
     def send_alert(self, record: "Record") -> None:
