@@ -66,9 +66,9 @@ def prompt_api_key(  # noqa: C901
     if anon_mode == "never":
         # Omit LOGIN_CHOICE_ANON as a choice if the env var is set to never
         choices.remove(LOGIN_CHOICE_ANON)
-    if jupyter or no_offline:
+    if (jupyter and not settings.login_timeout) or no_offline:
         choices.remove(LOGIN_CHOICE_DRYRUN)
-    if jupyter or no_create:
+    if (jupyter and not settings.login_timeout) or no_create:
         choices.remove(LOGIN_CHOICE_NEW)
 
     if jupyter and "google.colab" in sys.modules:
@@ -90,9 +90,14 @@ def prompt_api_key(  # noqa: C901
     elif len(choices) == 1:
         result = choices[0]
     else:
-        result = prompt_choices(choices, input_timeout=settings.login_timeout)
+        result = prompt_choices(
+            choices, input_timeout=settings.login_timeout, jupyter=jupyter
+        )
 
-    api_ask = "%s: Paste an API key from your profile and hit enter: " % log_string
+    api_ask = (
+        f"{log_string}: Paste an API key from your profile and hit enter, "
+        "or press ctrl+c to quit: "
+    )
     if result == LOGIN_CHOICE_ANON:
         key = api.create_anonymous_api_key()
 
@@ -102,9 +107,7 @@ def prompt_api_key(  # noqa: C901
         key = browser_callback(signup=True) if browser_callback else None
 
         if not key:
-            wandb.termlog(
-                "Create an account here: {}/authorize?signup=true".format(app_url)
-            )
+            wandb.termlog(f"Create an account here: {app_url}/authorize?signup=true")
             key = input_callback(api_ask).strip()
 
         write_key(settings, key, api=api)
@@ -114,9 +117,7 @@ def prompt_api_key(  # noqa: C901
 
         if not key:
             wandb.termlog(
-                "You can find your API key in your browser here: {}/authorize".format(
-                    app_url
-                )
+                f"You can find your API key in your browser here: {app_url}/authorize"
             )
             key = input_callback(api_ask).strip()
         write_key(settings, key, api=api)
@@ -124,6 +125,8 @@ def prompt_api_key(  # noqa: C901
     elif result == LOGIN_CHOICE_NOTTY:
         # TODO: Needs refactor as this needs to be handled by caller
         return False
+    elif result == LOGIN_CHOICE_DRYRUN:
+        return None
     else:
         # Jupyter environments don't have a tty, but we can still try logging in using
         # the browser callback if one is supplied.
@@ -149,9 +152,7 @@ def write_netrc(host, entity, key):
         normalized_host = urlparse(host).netloc.split(":")[0]
         if normalized_host != "localhost" and "." not in normalized_host:
             wandb.termerror(
-                "Host must be a url in the form https://some.address.com, received {}".format(
-                    host
-                )
+                f"Host must be a url in the form https://some.address.com, received {host}"
             )
             return None
         wandb.termlog(
@@ -159,7 +160,7 @@ def write_netrc(host, entity, key):
                 normalized_host, os.path.expanduser("~/.netrc")
             )
         )
-        machine_line = "machine %s" % normalized_host
+        machine_line = f"machine {normalized_host}"
         path = os.path.expanduser("~/.netrc")
         orig_lines = None
         try:
@@ -198,7 +199,7 @@ def write_netrc(host, entity, key):
 
 def write_key(settings, key, api=None, anonymous=False):
     if not key:
-        return
+        raise ValueError("No API key specified.")
 
     # TODO(jhr): api shouldn't be optional or it shouldnt be passed, clean up callers
     api = api or InternalApi()
