@@ -1,22 +1,26 @@
-#
+from typing import Any, Callable, Optional
+
 import wandb
 
 
-class PreInitObject(object):
-    def __init__(self, name):
+class PreInitObject:
+    def __init__(self, name: str, destination: Optional[Any] = None) -> None:
         self._name = name
 
-    def __getitem__(self, key):
+        if destination is not None:
+            self.__doc__ = destination.__doc__
+
+    def __getitem__(self, key: str) -> None:
         raise wandb.Error(
             'You must call wandb.init() before {}["{}"]'.format(self._name, key)
         )
 
-    def __setitem__(self, key, value):
+    def __setitem__(self, key: str, value: Any) -> Any:
         raise wandb.Error(
             'You must call wandb.init() before {}["{}"]'.format(self._name, key)
         )
 
-    def __setattr__(self, key, value):
+    def __setattr__(self, key: str, value: Any) -> Any:
         if not key.startswith("_"):
             raise wandb.Error(
                 "You must call wandb.init() before {}.{}".format(self._name, key)
@@ -24,7 +28,7 @@ class PreInitObject(object):
         else:
             return object.__setattr__(self, key, value)
 
-    def __getattr__(self, key):
+    def __getattr__(self, key: str) -> Any:
         if not key.startswith("_"):
             raise wandb.Error(
                 "You must call wandb.init() before {}.{}".format(self._name, key)
@@ -33,12 +37,14 @@ class PreInitObject(object):
             raise AttributeError()
 
 
-def PreInitCallable(name, destination=None):  # noqa: N802
-    def preinit_wrapper(*args, **kwargs):
+def PreInitCallable(  # noqa: N802
+    name: str, destination: Optional[Any] = None
+) -> Callable:
+    def preinit_wrapper(*args: Any, **kwargs: Any) -> Any:
         raise wandb.Error("You must call wandb.init() before {}()".format(name))
 
     preinit_wrapper.__name__ = str(name)
     if destination:
-        preinit_wrapper.__wrapped__ = destination
+        preinit_wrapper.__wrapped__ = destination  # type: ignore
         preinit_wrapper.__doc__ = destination.__doc__
     return preinit_wrapper
